#include "BigNumHelpers.h"
#include <iostream>

using namespace std ;

/* See header file for documentation */
void mult2Unsigned( unsigned a, unsigned b, 
				    unsigned &result, unsigned &carry ) {
	// post: carry' * 2^32 + result' = a * b
	unsigned aLower = a & 0xFFFF ;
	unsigned aUpper = a>>16 ;
	unsigned bLower = b & 0xFFFF ;
	unsigned bUpper = b>>16 ;
	
	unsigned c = aLower * bLower ;
	unsigned d = aLower * bUpper ;
	unsigned e = aUpper * bLower ;
	unsigned f = aUpper * bUpper ;
	unsigned ds = d << 16 ; // (the lower half of d) * 2^16
	unsigned es = e << 16 ; // (the lower half of e) * 2^16
	
	result = c + ds + es ;

	int msb_count = 0 ;
	if( c >> 31 ) msb_count += 1 ;
	if( ds >> 31 ) msb_count += 1 ;
	if( es >> 31 ) msb_count += 1 ;
	unsigned result_msb = result >> 31 ;
	
	if( msb_count == 0 ) carry = 0 ;
	else if( msb_count == 1 ) carry = ! result_msb ;
	else carry = 1 ;
	
	carry += f + (d >> 16) + (e >> 16);
}

/* See header file for documentation */
void delList( BigNumNode *&head ) {
	// Invariant: All nodes originally between the original value of head
	//            and the current value of head have been deleted.
	while( head != 0 ) {
		BigNumNode *temp = head ;
		head = head->next ;
		delete temp ; }
}

/* See header file for documentation */
void printList( BigNumNode *p ) {
	cout << "[" ;
	// Invariant: All nodes originally between the original value of head
	//            and the current value of head have been printed to cout.
	while( p != 0 ) {
		cout << p->digit ;
		p = p->next ;
		if( p != 0 ) cout << ", " ; }
	cout << "]" ;
} 

/* See header file for documentation */
void makeNode( BigNumNode *&link, unsigned digit, BigNumNode *next, bool &ok) {
	link = new BigNumNode ;
	if( link == 0 ) {
		// out of space
		ok = false ; }
	else {
		link->digit = digit ;
		link->next = next ;
		ok = true ; }
}

/* See header file for documentation */
void copyList( BigNumNode *p, BigNumNode *&newHead, bool &ok ) {
	newHead = 0 ;
	BigNumNode **linkP = & newHead ;
	// Invariant: All nodes between the original value of p 
	//            and the current value of p have been copied
	//            and the copies lie between newHead and *linkP.
	while( p != 0 ) {
		makeNode( *linkP, p->digit, 0, ok ) ;
		if( !ok ) { delList(newHead) ; return ; }
		p = p->next ;
		linkP = & (*linkP)->next ; }
}
