
#ifndef BIG_NUM_HELPERS
#define BIG_NUM_HELPERS

#include "BigNumNode.h"

/** Compute the 64 bit product of two 32 bit numbers.
  * param a Input. A 32 bit number
  * param b Input. A 32 bit number
  * param result Output. The lower 32 bits of the product a times b
  * param carry Output. The upper 32 bits of the product a times b
  * postcondition: carry' * 2^32 + result' = a * b
  */
void mult2Unsigned( unsigned a, unsigned b, 
				    unsigned &result, unsigned &carry ) ;


/** Delete all the nodes of the list headed by head and set head to null */
void delList( BigNumNode *&head ) ;

/** Print each the list headed by p to cout */
void printList( BigNumNode *p ) ;

/** Allocate a single node.
  * If the node can be allocated, then its address is stored in
  * link, its fields are initialized by digit and next, and ok is set to true.
  * If space can not be found, link is set to null, and ok is set to false
  */
void makeNode( BigNumNode *&link, unsigned digit, BigNumNode *next, bool &ok) ;

/** Make a copy of a list.
  * Makes a copy of each node in the list headed by p and
  * links them together into a new list headed by newHead.
  * If this can not be done because of space restrictions, ok
  * will be set to false, otherwise ok will be set to true.
  */
void copyList( BigNumNode *p, BigNumNode *&newHead, bool &ok ) ;
	
#endif