#include <iostream>
#include <new>

using namespace std ;

#include "BigNum.h"
#include "BigNumHelpers.h"

bool eqList( BigNumNode* a, BigNumNode*b ) {
    return a==0 && b==0
        || a!=0 && b!=0 && a->digit == b->digit && eqList( a->next, b->next) ;
}

void makeList( unsigned *a, BigNumNode* &head, bool &ok ) {
    head = 0 ;
	BigNumNode **linkP = & head ;
	int n = *a ;
    ok = true ;
    for(int i=1 ; i <= n ; ++i ) {
		makeNode( *linkP, a[i], 0, ok ) ;
        if( !ok ) { delList( head ) ; return; }
		linkP = & (*linkP)->next ;
	}
}

void testAdd1(  unsigned digit ,
			    unsigned *originalValueArray,
                unsigned *expectedResultArray,
                bool &pass )
{
    bool ok ;
    BigNumNode * value ;
    makeList( originalValueArray, value, ok ) ;
    if( !ok ) {pass=true ; return ; }
    BigNumNode *  expectedResult;
    makeList( expectedResultArray, expectedResult, ok ) ;
    if( !ok ) {pass=true ; return ; }
    
    ok = false ;

    cout << "Adding " << digit << " to " ; printList( value ) ; cout << endl ; 
    cout << "Expected : " ; printList( expectedResult ) ; cout << endl ; 
    cout << "Result   : " ;
    add1( digit, value, ok ) ;
    if( ok ) { printList( value ) ; cout << endl ; }
	else cout << "ok false" << endl ;

    pass = ok && eqList( value, expectedResult ) ;

    delList( value ) ;
    delList( expectedResult ) ;
}

void testAdd(  unsigned *addendArray,
			    unsigned *originalValueArray,
                unsigned *expectedResultArray,
                bool &pass )
{
    bool ok ;
	BigNumNode * addend ;
	makeList( addendArray, addend, ok ) ;
	if( !ok ) {pass=true ; return ; }

    BigNumNode * value ;
    makeList( originalValueArray, value, ok ) ;
    if( !ok ) {pass=true ; return ; }
    
	BigNumNode *  expectedResult;
    makeList( expectedResultArray, expectedResult, ok ) ;
    if( !ok ) {pass=true ; return ; }
    
    ok = false ;

    cout << "Adding " ; printList( addend ) ; cout << endl ; 
	cout << " to    " ; printList( value ) ; cout << endl ; 
    cout << "Expected : " ; printList( expectedResult ) ; cout << endl ; 
    cout << "Result   : " ;
    add( addend, value, ok ) ;
    if( ok ) { printList( value ) ; cout << endl ; }
	else cout << "ok false" << endl ;

    pass = ok && eqList( value, expectedResult ) ;

	delList( addend ) ;
    delList( value ) ;
    delList( expectedResult ) ;
}

void testMult1( unsigned digit,
				unsigned *multiplicand_array,
                unsigned *product_array,
                bool &pass )
{
    bool ok ;
    BigNumNode * multiplicand ;
    makeList( multiplicand_array, multiplicand, ok ) ;
    if( !ok ) {pass=true ; return ; }
    BigNumNode * product ;
    makeList( product_array, product, ok ) ;
    if( !ok ) {pass=true ; return ; }
    
    BigNumNode *test_prod ;
    ok = false ;

    cout << "Multiplying " << digit << " by " ; printList( multiplicand ) ; cout << endl ; 
    cout << "Expected : " ; printList( product ) ; cout << endl ; 
    cout << "Result   : " ;
    mult1( digit, multiplicand,  test_prod, ok ) ;
    if( ok ) { printList( test_prod ) ; cout << endl ; }
	else cout << "ok false" << endl ;

    pass = ok && eqList( test_prod, product ) ;

    delList( multiplicand ) ;
    delList( product ) ;
    delList( test_prod ) ;
}

void testMult(   unsigned *multiplicandArray,
			    unsigned *multiplierArray,
                unsigned *expectedResultArray,
                bool &pass )
{
    bool ok ;
	BigNumNode * multiplicand ;
	makeList( multiplicandArray, multiplicand, ok ) ;
	if( !ok ) {pass=true ; return ; }

    BigNumNode * multiplier ;
    makeList( multiplierArray, multiplier, ok ) ;
    if( !ok ) {pass=true ; return ; }
    
	BigNumNode *  expectedResult;
    makeList( expectedResultArray, expectedResult, ok ) ;
    if( !ok ) {pass=true ; return ; }
    
    ok = false ;

    cout << "Multiplying " ; printList( multiplicand ) ; cout << endl ; 
	cout << " by         " ; printList( multiplier ) ; cout << endl ; 
    cout << "Expected : " ; printList( expectedResult ) ; cout << endl ; 
    cout << "Result   : " ;
	BigNumNode *product ;
    mult( multiplicand, multiplier, product, ok ) ;
    if( ok ) { printList( product ) ; cout << endl ; }
	else cout << "ok false" << endl ;

    pass = ok && eqList( product, expectedResult ) ;

	delList( multiplicand ) ;
    delList( multiplier ) ;
    delList( product ) ;
    delList( expectedResult ) ;
}

int testCount() { return 29 ; }

// doTest  --- do one test //
/////////////////////////////
void doTest( int testNumber, int &result ) {
    try {
		static unsigned zero[] = { 0 } ;
        bool pass = true ;
        bool success = false ;
        cout << "Test " << testNumber << "\n" ;
        switch( testNumber ) {
        case 0 : {
            // Add 0 to 0
            testAdd1(0, zero, zero, pass ) ;
        } break ;
        case 1 : {
			// Add a large digit to zero
			static unsigned ev[] = { 1, 0xFFFFFFFFu } ;
			testAdd1( 0xFFFFFFFFu, zero, ev, pass ) ;
        } break ;
        case 2 :{
			// No carry
			static unsigned v[] = { 1, 0xFFFFFFF5u } ;
			static unsigned ev[]= { 1, 0xFFFFFFFFu } ;
			testAdd1( 10, v, ev, pass ) ;
        } break ;
        case 3 : {
			// Carry
			static unsigned v[] = { 1, 0xFFFFFFFAu } ;
			static unsigned ev[]= { 2, 0xFFFFFFF5u, 1 } ;
			testAdd1( 0xFFFFFFFBu, v, ev, pass ) ;
        } break ;
        case 4 : {
			// Repeated Carry
			static unsigned v[] = { 4, 0xFFFFFFFAu, 0xFFFFFFFFu, 0xFFFFFFFFu, 0xFFFFFFFFu  } ;
			static unsigned ev[]= { 5, 4, 0, 0, 0, 1 } ;
			testAdd1( 10, v, ev, pass ) ;
        } break ;
        case 5 : {
			// Add 0 to 0
			testAdd( zero, zero, zero, pass ) ;
        } break ;
        case 6 : {
			// Add zero to nonzero
			static unsigned a[] = { 4, 0xFFFFFFFAu, 0xFFFFFFFFu, 0xFFFFFFFFu, 0xFFFFFFFFu  } ;
			testAdd( zero, a, a, pass ) ;
        } break ;
        case 7 : {
			// Add nonzero to zero
			static unsigned a[] = { 4, 0xFFFFFFFAu, 0xFFFFFFFFu, 0xFFFFFFFFu, 0xFFFFFFFFu  } ;
			testAdd( a, zero, a, pass ) ;
        } break ;
        case 8 : {
			// Add numbers same size with no carry
			static unsigned a[] = { 4, 1, 2, 3, 4  } ;
			static unsigned c[] = { 4, 2, 4, 6, 8  } ;
			testAdd( a, a, c, pass ) ;
        } break ;
        case 9 : {
			// Add numbers same size with no carry
			static unsigned a[] = { 4, 1, 2, 3, 4  } ;
			static unsigned b[] = { 2, 0xFFFFFFFEu, 0xFFFFFFFDu } ;
			static unsigned c[] = { 4, 0xFFFFFFFFu, 0xFFFFFFFFu, 3, 4 } ;
			testAdd( a, b, c, pass ) ;
        } break ;
        case 10 : {
			// Add numbers same size with no carry
			static unsigned a[] = { 4, 1, 2, 3, 4  } ;
			static unsigned b[] = { 2, 0xFFFFFFFEu, 0xFFFFFFFDu } ;
			static unsigned c[] = { 4, 0xFFFFFFFFu, 0xFFFFFFFFu, 3, 4 } ;
			testAdd( b, a, c, pass ) ;
        } break ;
        case 11 : {
			// Add numbers of different sizes with no final carry 
			static unsigned a[] = { 5, 0, 0, 0xFFFFFFFFu, 0xFFFFFFFFu,   1  } ;
			static unsigned b[] = { 4, 0, 0,           1,           1 } ;
			static unsigned c[] = { 5, 0, 0,           0,           1,   2 } ;
			testAdd( a, b, c, pass ) ;
        } break ;
        case 12 : {
			// Add numbers of different sizes with no final carry 
			static unsigned a[] = { 5, 0, 0, 0xFFFFFFFFu, 0xFFFFFFFFu,   1  } ;
			static unsigned b[] = { 4, 0, 0,           1,           1 } ;
			static unsigned c[] = { 5, 0, 0,           0,           1,   2 } ;
			testAdd( b, a, c, pass ) ;
        } break ;
        case 13 : {
			// Add numbers of different sizes with final carry 
			static unsigned a[] = { 3, 0xFFFFFFFFu, 0xFFFFFFFFu, 0xFFFFFFFFu  } ;
			static unsigned b[] = { 2, 1, 1 } ;
			static unsigned c[] = { 4, 0, 1, 0, 1 } ;
			testAdd( a, b, c, pass ) ;
        } break ;
        case 14 : {
			// Add numbers of different sizes with final carry 
			static unsigned a[] = { 3, 0xFFFFFFFFu, 0xFFFFFFFFu, 0xFFFFFFFFu  } ;
			static unsigned b[] = { 2, 1, 1 } ;
			static unsigned c[] = { 4, 0, 1, 0, 1 } ;
			testAdd( b, a, c, pass ) ;
		} break ;
        case 15 : {
			// Mult1 of 0 times 0
			testMult1( 0, zero, zero, pass ) ;
		} break ;
        case 16 : {
			// Mult1 of nonzero times 0
			testMult1( 1000000u, zero, zero, pass ) ;
		} break ;
        case 17 : {
			// Mult1 of 0 times something non zero
			static unsigned a[] = { 3, 0xFFFFFFFFu, 0xFFFFFFFFu, 0xFFFFFFFFu  } ;
			testMult1( 0, a, zero, pass ) ;
		} break ;
        case 18 : {
			// Mult1 of 1 times something
			static unsigned a[] = { 3, 0xFFFFFFFFu, 0xFFFFFFFFu, 0xFFFFFFFFu  } ;
			testMult1( 1, a, a, pass ) ;
		} break ;
        case 19 : {
			// Mult1 with no carry
			static unsigned a[] = { 3, 0,    1000,    1000000  } ;
			static unsigned c[] = { 3, 0, 1000000, 1000000000  } ;
			testMult1( 1000, a, c, pass ) ;
		} break ;
        case 20 : {
			// Mult1 with carry
			static unsigned a[] = { 3, 0,    1000000,        1  } ;
			static unsigned c[] = { 3, 0, 3567587328,  1000232  } ;
			testMult1( 1000000, a, c, pass ) ;
		} break ;
        case 21 : {
			// Mult1 with carry out
			static unsigned a[] = { 3, 0,    1000000,    1000000  } ;
			static unsigned c[] = { 4, 0, 3567587328, 3567587560, 232  } ;
			testMult1( 1000000, a, c, pass ) ;
		} break ;
        case 22 : {
			// Mult of 0 times 0
			testMult( zero, zero, zero, pass ) ;
		} break ;
        case 23 : {
			// Mult of nonzero times 0
			static unsigned a[] = { 3, 0xFFFFFFFFu, 0xFFFFFFFFu, 0xFFFFFFFFu  } ;
			testMult( a, zero, zero, pass ) ;
		} break ;
        case 24 : {
			// Mult of 0 times nonzero
			static unsigned a[] = { 3, 0xFFFFFFFFu, 0xFFFFFFFFu, 0xFFFFFFFFu  } ;
			testMult( zero, a, zero, pass ) ;
		} break ;
        case 25 : {
			// A multiplication that amounts to a shift
			static unsigned a[] = { 3, 0xFFFFFFFFu, 0xFFFFFFFFu, 0xFFFFFFFFu  } ;
			static unsigned b[] = { 3,           0,           0,           1  } ;
			static unsigned c[] = { 5, 0, 0, 0xFFFFFFFFu, 0xFFFFFFFFu, 0xFFFFFFFFu  } ;
			testMult( a, b, c, pass ) ;
		} break ;
        case 26 : {
			// A multiplication that has some carries
			static unsigned a[] = { 3, 0xFFFFFFFFu, 0xFFFFFFFFu, 0xFFFFFFFFu  } ;
			static unsigned b[] = { 3,           2,           2,           2  } ;
			static unsigned c[] = { 6, 0xFFFFFFFEu, 0xFFFFFFFDu, 0xFFFFFFFDu, 1, 2, 2 } ;
			testMult( a, b, c, pass ) ;
		} break ;
        case 27 : {
			// Lots of carries.
			static unsigned a[] = { 3, 0xFFFFFFFFu, 0xFFFFFFFFu, 0xFFFFFFFFu  } ;
			static unsigned c[] = { 6, 1, 0, 0, 0xFFFFFFFEu, 0xFFFFFFFFu, 0xFFFFFFFFu } ;
			testMult( a, a, c, pass ) ;
		} break ;
        case 28 : {
			// Multiplying some big numbers
			static unsigned a[] = { 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1000  } ;
			static unsigned b[] = {  9, 0, 0, 0, 0, 0, 0, 0, 0, 1000 } ;
			static unsigned c[] = { 18, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1000000 } ;
			testMult( a, b, c, pass ) ;
		} break ;
        default : pass = false ;
        } 

        if( pass ) {
            result = 0 ; }
        else {
            result = 1 ; } }
    catch( ... ) {
        cerr << "Sorry an unanticipated exception occurred\n" ;
        result = 1 ; }
}